unit Globals;

//  ////////////////////////////////////////////////////////////////////////////
//
//  Copyright (C) 2004 .. 2006 Jan Punter
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  ////////////////////////////////////////////////////////////////////////////
//
//    lupd : 2006-03-26
//
//  ////////////////////////////////////////////////////////////////////////////


interface

Uses

  Windows, SysUtils, Messages, StdCtrls, Forms, Dialogs, Registry, ShellApi;


const

  // Midi definitions

  CNoteOff         = $80;
  CNoteOn          = $90;
  CKeyPressure     = $a0;
  CControlChange   = $b0;
  CProgramChange   = $c0;
  CChannelPressure = $d0;
  CPitchBend       = $e0;
  CSysEx           = $f0;

  
// Our name and path stuff

  Function  ApplicationPath    : String;
  Function  ApplicationPathURL : String;
  Function  ApplicationFileName: String;
  Function  IniFileName        : String;
  Function  LogFileName        : String;
  Function  TestFileName       : String;
  Function  HelpFileName       : String;
  Function  FileNameToURL( Const aFileName: String): String;
  Function  Webbize( Const aFileName: String): String;

  Function  ProgramAssociationFor( anExt: String): String;

  Function  BoolToStr   ( aValue: Boolean ): String;
  Function  ByteToHex   ( aValue: Cardinal): String;
  Function  WordToHex   ( aValue: Cardinal): String;
  Function  LongToHex   ( aValue: Cardinal): String;
  Function  LongToBinary( aValue: Cardinal; aLength: Integer = 32): String;
  Function  LongToBase  ( aValue: Cardinal; aBase: Cardinal): String;
  Function  HexToByte   ( Const aValue: String): Cardinal;
  Function  HexToWord   ( Const aValue: String): Cardinal;
  Function  HexToLong   ( Const aValue: String): Cardinal;
  Function  ByteToAscii ( aValue: Byte): Char;
  Function  Indent      ( anInd: Integer): String;
  function  IndStr      ( anInd: Integer; aMsg: string): string;
  function  IndFmt      ( anInd: Integer; const aFmt: string; const anArgs: array of const): string;
  Function  TrimStr     ( Const S: String; aLength: Integer): String;
  Function  ExtendStr   ( Const S: String; aLength: Integer): String;

  Function  IntegerToNoteStr( aNoteValue: Integer): String;

  Function  CrlfEncode( const S: string): string;
  Function  CrlfDecode( const S: string): string;

  function  ByteSwap( aWord: Word): Word;


// System stuff

  Function  CleanFileName( Const aFileName : String): String;
  // Returns filename only, path and extension are stripped off.

  Function  IsDottedIp( Const S: String): Boolean;

// Miscellaneous

  Procedure NotImplemented( Const aMsg: String);
  Procedure Noop;

  Function  GetCompanyName      : String;
  Function  GetFileDescription  : String;
  Function  GetFileVersion      : String;
  Function  GetInternalName     : String;
  Function  GetLegalCopyRight   : String;
  Function  GetLegalTradeMarks  : String;
  Function  GetOriginalFileName : String;
  Function  GetProductName      : String;
  Function  GetProductVersion   : String;
  Function  GetComments         : String;
  Function  GetEmail            : String;
  Function  GetWww              : String;
  Function  GetPhone            : string;
  function  GetFax              : string;
  function  GetAddress1         : string;
  function  GetAddress2         : string;

  Procedure CopyFile( Const aSrcName, aDstName: String);
  Procedure ViewFile( WindowHandle: HWND; Const aFileName: String);
  Procedure RemoveFolderContents( Const aName: String);
  Procedure RemoveFolder( Const aName: String);
  Function  MakeRelativePath( Const aReference, aFileName: String): String;
  Function  MakeRelativeUrl( Const aReference, aFileName: String): String;
  Procedure StringToFile( Const aFileName, aValue: String);
  Procedure StringToFileAppend( Const aFileName, aValue: String);

  Function  QueryUserName: String;
  Procedure ExploreFolder( aWindowHandle: THandle; Const aFolderName: String);
  Procedure ShellOpen( aWindowHandle: THandle; Const aFileName: String);

implementation


// Our name and path stuff

Var

  strApplicationPath     : String;
  strApplicationPathURL  : String;
  strApplicationFileName : String;
  strIniFileName         : String;
  strLogFileName         : String;
  strTstFileName         : String;
  strHlpFileName         : String;
  strIndent              : String;


  Function ApplicationPath: String;
  Begin
    Result := ExcludeTrailingPathDelimiter( strApplicationPath);
  End;


  Function ApplicationPathURL : String;
  Begin
    Result := strApplicationPathURL;
  End;


  Function ApplicationFileName: String;
  Begin
    Result := strApplicationFileName;
  End;


  Function IniFileName: String;
  Begin
    Result := strIniFileName;
  End;


  Function LogFileName: String;
  Begin
    Result := strLogFileName;
  End;


  Function TestFileName: String;
  Begin
    Result := strTstFileName;
  End;


  Function HelpFileName: String;
  Begin
    Result := strHlpFileName;
  End;


  Function FileNameToURL( Const aFileName: String): String;
  Var
    i : Integer;
  Begin
    Result := 'file:///';
    For i := 1 To Length( aFileName) Do
      Case aFileName[ i] Of
        ':' : Result := Result + '|';
        '\' : Result := Result + '/';
        Else  Result := Result + aFileName[ i]
      End;
  End;


  Function Webbize( Const aFileName: String): String;
  Var
    i : Integer;
  Begin
    Result := '';
    For i := 1 To Length( aFileName) Do
      Case aFileName[ i] Of
        ':' : Result := Result + '|';
        '\' : Result := Result + '/';
        ' ' : Result := Result + '_';
        Else  Result := Result + LowerCase( aFileName[ i])
      End;
  End;


  Function ProgramAssociationFor( anExt: String): String;
  Var
    Reg : TRegistry;
    S   : String;
  Begin
    S := '';
    Reg := TRegistry.Create;
    Reg.RootKey := HKEY_CLASSES_ROOT;
    If Reg.OpenKey( '.' + anExt + '\Shell\open\command', false)
    Then Begin
      S := Reg.ReadString( '');
      Reg.CloseKey;
    End
    Else Begin
      If Reg.OpenKey( '.' + anExt, False)
      Then Begin
        S := Reg.ReadString( '');
        Reg.CloseKey;
        If S <> ''
        Then Begin
          If Reg.OpenKey( S + '\Shell\Open\Command', False)
          Then S := Reg.ReadString( '');
          Reg.CloseKey;
        End;
      End;
    End;
    If Pos( '%', S) > 0
    Then Begin
      If Pos( '%1', S) > 0
      Then Delete( S, Pos( '%1', S), 2)
      Else Delete( S, Pos( '%', S), 1);
    End;
    While (
        ( Length( S) > 0) And
        (
          ( S[ Length( S)] = ' ') Or
          ( S[ Length( S)] = '"')
        )
      ) Do
      Delete( S, Length( S), 1);
    Result := S;
  End;


  Function  BoolToStr( aValue : Boolean): String;
  Begin
    Case aValue Of
      False : Result := 'False';
      True  : Result := 'True';
    End;
  End;


  Function  ByteToHex( aValue: Cardinal): String;
  Begin
    Result := Format( '%0.2x', [ aValue And $00ff]);
  End;


  Function  WordToHex( aValue: Cardinal): String;
  Begin
    Result := Format( '%0.4x', [ aValue And $ffff]);
  End;


  Function  LongToHex( aValue: Cardinal): String;
  Begin
    Result := Format( '%0.4x.%0.4x', [ aValue And $ffff0000 Shr 16, aValue And $ffff]);
  End;


  Function  LongToBinary( aValue: Cardinal; aLength: Integer = 32): String;
  Begin
    Result := '';
    While aValue <> 0 Do
    Begin
      If aValue And 1 = 1
      Then Result := '1' + Result
      Else Result := '0' + Result;
      aValue := aValue Shr 1;
    End;
    If Result = ''
    Then Result := '0';
    while Length( Result) < aLength
    do Result := '0' + Result;
  End;


  Function  LongToBase( aValue: Cardinal; aBase: Cardinal): String;

    Function ToDigit( aValue, aBase: Cardinal): String;
    Begin
      Assert( aValue < aBase, 'Digit conversion error, Value >= Base');
      If aBase <= 36
      Then
        If aValue > 10
        Then Result := Char( aValue + Ord( '0') + 7)
        Else Result := Char( aValue + Ord( '0'))
      Else Result := Format( '<&%d>', [ aValue]);
    End;

  Begin
    Assert( aBase > 1, 'Number base must be larger than 1');
    Result := '';
    While aValue <> 0 Do
    Begin
      Result := ToDigit( aValue Mod aBase, aBase) + Result;
      aValue := aValue Div aBase;
    End;
    If Result = ''
    Then Result := '0';
  End;


  Function  HexToByte( Const aValue: String): Cardinal;
  Begin
    Result := StrToInt( Format( '$%s', [ aValue]));
  End;


  Function  HexToWord( Const aValue: String): Cardinal;
  Begin
    Result := StrToInt( Format( '$%s', [ aValue]));
  End;


  Function  HexToLong( Const aValue: String): Cardinal;
  Var
    S : String;
    p : Integer;
  Begin
    S := aValue;
    p := Pos( '.', S);
    While p > 0 Do
    Begin
      S := Copy( S, 1, p - 1) + Copy( S, p + 1, Length( S));
      p := Pos( '.', S);
    End;
    Result := StrToInt( Format( '$%s', [ S]));
  End;


  Function  ByteToAscii( aValue: Byte): Char;
  Begin
    If ( aValue < 32) Or ( aValue >= 127)
    Then Result := '.'
    Else Result := Char( aValue);
  End;


  Function  Indent( anInd: Integer): String;
  Begin
    Result := Copy( strIndent, 1, 2 * anInd);
  End;

  function  IndStr( anInd: Integer; aMsg: string): string;
  begin
    Result := Format( '%s%s', [ Indent( anInd), aMsg]);
  end;

  function  IndFmt( anInd: Integer; const aFmt: string; const anArgs: array of const): string;
  begin
    Result := IndStr( anInd, Format( aFmt, anArgs));
  end;

  Function  TrimStr( Const S: String; aLength: Integer): String;
  Begin
    Result := Copy( ExtendStr( S, aLength), 1, aLength); // Procrustrus trimming.
  End;


  Function  ExtendStr( Const S: String; aLength: Integer): String;
  Begin
    Result := S;
    While Length( Result) < aLength Do
      Result := Result + ' ';
  End;


  Function  IntegerToNoteStr( aNoteValue: Integer): String;
  Const
    Notes  : Array[ 0 .. 11] Of Char = 'CCDDEFFGGAAB';
    Sharps : Array[ 0 .. 11] Of Char = ' # #  # # # ';
  Var
    i : Integer;
  Begin
    If ( aNoteValue < 0) Or ( aNoteValue > 127)
    Then Result := 'inv'
    Else Begin
      i := aNoteValue Mod 12;
      Result := Format( '%s%d%s', [ Notes[ i], ( aNoteValue Div 12) - 5, Sharps[ i]]);
    End;
  End;


  Procedure InitModule;
  Var
    S : String;
    p : Integer;
  Begin
    S := Application.ExeName;
    strApplicationFileName := ExtractFileName( S);
    strApplicationPath     := ExtractFilePath( S);
    strApplicationPathURL  := FileNameToURL( strApplicationPath);
    p := Pos( '.', strApplicationFileName);
    If p = 0
    Then Begin
      strIniFileName := strApplicationPath + strApplicationFileName + '.INI';
      strLogFileName := strApplicationPath + strApplicationFileName + '.LOG';
      strTstFileName := strApplicationPath + strApplicationFileName + '.TST';
      strHlpFileName := strApplicationPath + strApplicationFileName + '.HTM';
    End
    Else Begin
      strIniFileName := strApplicationPath + Copy( strApplicationFileName, 0, p - 1) + '.INI';
      strLogFileName := strApplicationPath + Copy( strApplicationFileName, 0, p - 1) + '.LOG';
      strTstFileName := strApplicationPath + Copy( strApplicationFileName, 0, p - 1) + '.TST';
      strHlpFileName := strApplicationPath + Copy( strApplicationFileName, 0, p - 1) + '.HTM';
    End;
    strIndent := ' ';
    While Length( strIndent) < 256 Do
      strIndent := strIndent + strIndent;
  End;


  Function  CrlfEncode( const S: string): string;
  var
    i : Integer;
  begin
    Result := '';
    for i := 1 to Length( S) do
    begin
      case S[ i] of
        #$0d : Result := Result + '^M';
        #$0a : Result := Result + '^J';
        ^    : Result := Result + '^^';
        else   Result := result + S[ i];
      end;
    end;
  end;

  Function  CrlfDecode( const S: string): string;
  var
    i     : Integer;
    State : (
      stNormal,
      stCaret
    );
  begin
    Result := '';
    State  := stNormal;
    for i := 1 to Length( S) do
    begin
      case State of
        stNormal : begin
            case S[ i] of
              '^' : State := stCaret;
              else Result := Result + S[ i];
            end;
          end;
        stCaret : begin
            case S[ i] of
              'J' : Result := Result + #$0a;
              'M' : Result := Result + #$0d;
              '^' : Result := Result + '^';
              else Result := Result + S[ i];
            end;
            State := stNormal;
          end;
      end;
    end;
  end;

  function  ByteSwap( aWord: Word): Word;
  begin
    Result := ( 256 * ( aWord and $ff)) + (( aWord shr 8) and $ff);
  end;

  Function  CleanFileName( Const aFileName : String): String;
  Var
    i : Integer;
  Begin
    Result := ExtractFileName( aFileName);
    i := Pos( '.', Result);
    If i > 0
    Then Result := Copy( Result, 1, i - 1);
  End;

  Function  IsDottedIp( Const S: String): Boolean;

    Function IsValidInteger( Const N: String): Boolean;
    Var
      aCode : Integer;
      aNum  : Integer;
    Begin
      Val( N, aNum, aCode);
      Result := ( aCode = 0) And ( aNum >= 0) And ( aNum < 256);
    End;

  Const
    Digits : Set Of Char = [ '0' .. '9'];
  Var
    p      : Integer;
    N      : String;
    DotCnt : Integer;
  Begin
    p      := 1;
    N      := '';
    DotCnt := 0;
    Result := True;
    While ( p <= Length( S)) And Result Do
    Begin
      If S[ p] In Digits
      Then Begin
        N := N + S[ p];
        If Length( N) > 3
        Then Begin
          Result := False;
          Break;
        End;
      End
      Else If S[ p] = '.'
      Then Begin
        Result := IsValidInteger( N);
        Inc( DotCnt);
        N := '';
      End
      Else Begin
        Result := False;
        Break;
      End;
      Inc( p);
    End;
    If Result
    Then Result := ( DotCnt = 3) And IsValidInteger( N);
  End;



// Miscellaneous

  Procedure NotImplemented( Const aMsg: String);
  Begin
    MessageDlg(
      Format(
        'Not implemented : %s',
        [ aMsg]
      ),
      mtInformation,
      [ mbOk, mbCancel],
      0
    );
  End;

  Procedure Noop;
  Begin
  End;

// Version information support

Const

  InfoNum  = 16;

Var

  FileInfo : Array[ 1 .. InfoNum] Of String = (
    'Unknown',
    'Unknown',
    'Unknown',
    'Unknown',
    'Unknown',
    'Unknown',
    'Unknown',
    'Unknown',
    'Unknown',
    'Unknown',
    'Unknown',
    'Unknown',
    'Unknown',
    'Unknown',
    'Unknown',
    'Unknown'
  );

  Function  GetCompanyName: String;
  Begin
    Result := FileInfo[ 1];
  End;

  Function  GetFileDescription: String;
  Begin
    Result := FileInfo[ 2];
  End;

  Function  GetFileVersion: String;
  Begin
    Result := FileInfo[ 3];
  End;

  Function  GetInternalName: String;
  Begin
    Result := FileInfo[ 4];
  End;

  Function  GetLegalCopyRight: String;
  Begin
    Result := FileInfo[ 5];
  End;

  Function  GetLegalTradeMarks: String;
  Begin
    Result := FileInfo[ 6];
  End;

  Function  GetOriginalFileName: String;
  Begin
    Result := FileInfo[ 7];
  End;

  Function  GetProductName: String;
  Begin
    Result := FileInfo[ 8];
  End;

  Function  GetProductVersion: String;
  Begin
    Result := FileInfo[ 9];
  End;

  Function  GetComments: String;
  Begin
    Result := FileInfo[ 10];
  End;

  Function  GetPhone: string;
  begin
    Result := FileInfo[ 11];
  end;

  function  GetFax: string;
  begin
    Result := FileInfo[ 12];
  end;

  function  GetAddress1: string;
  begin
    Result := FileInfo[ 13];
  end;

  function  GetAddress2: string;
  begin
    Result := FileInfo[ 14];
  end;

  Function  GetEmail: String;
  Begin
    Result := FileInfo[ 15];
  End;

  Function  GetWww: String;
  Begin
    Result := FileInfo[ 16];
  End;


{$WARN UNSAFE_TYPE OFF}
{$WARN UNSAFE_CODE OFF}
{$WARN UNIT_PLATFORM   OFF}
{$WARN SYMBOL_PLATFORM OFF}

  Procedure   CopyFile( Const aSrcName, aDstName: String);
  Var
    aFile : File Of Byte;
    bFile : File Of Byte;
    aSize : Integer;
    aMem  : Pointer;
  Begin
    AssignFile( aFile, aSrcName);
    AssignFile( bFile, aDstName);
    Try
      Reset( aFile);
      aSize := FileSize( aFile);
      GetMem( aMem, aSize);
      Try
        BlockRead( aFile, aMem^, aSize);
        Rewrite( bFile);
        Try
          BlockWrite( bFile, aMem^, aSize);
        Finally
          CloseFile( bFile);
        End
      Finally
        FreeMem( aMem, aSize);
      End;
    Finally
      CloseFile( aFile);
    End;
  End;

  Procedure   ViewFile( WindowHandle: HWND; Const aFileName: String);
  Begin
    ShellExecute(
      WindowHandle,
      Nil,
      PChar( aFileName),
      Nil,
      Nil,
      SW_NORMAL
    );

  End;

  Procedure   RemoveFolderContents( Const aName: String);
  Var
    aSearchRec  : TSearchRec;
    SrchRes     : Integer;
    FName       : String;
  Begin
    // Remove all files andd folders from the directory aName
    Try
      SrchRes := SysUtils.FindFirst( aName + '\*.*', faAnyFile, aSearchRec);
      While SrchRes = 0 Do
      Begin
        Application.ProcessMessages;

        With aSearchRec Do
        Begin
          FName := FindData.cFileName;
          If Attr And faDirectory = 0
          Then DeleteFile( aName + '\' + FName)      // a file, delete it
          Else
            If
              ( FName <> '.')  And                   // Can't delete virtual folders
              ( FName <> '..')
            Then RemoveFolder( aName + '\' + FName); // a folder, recurse
        End;
        SrchRes := SysUtils.FindNext( aSearchRec);
      End;
    Finally
      FindClose( aSearchRec);
    End;
  End;

  Procedure   RemoveFolder( Const aName: String);
  Begin
    // First be sure that the directory aName has no files in it
    RemoveFolderContents( aName);
    // Then be sure that aName is not the current directory
    If aName[ Length( aName)] = '\'
    Then SetCurrentDir( ExtractFilePath( Copy( aName, 1, Length( aName) - 1)))
    Else SetCurrentDir( ExtractFilePath( aName));
    // Finally delete the directory aName
    RmDir( aName);
  End;

  Function  MakeRelativePath( Const aReference, aFileName: String): String;
  Begin
    If aReference[ Length( aReference)] = '\'
    Then Result := aReference
    Else Result := aReference + '\';
    Result := ExtractRelativePath( Result, aFileName);
  End;

  Function  MakeRelativeUrl( Const aReference, aFileName: String): String;
  Begin
    Result := Webbize( MakeRelativePath( aReference, aFileName));
  End;

  Procedure StringToFile( Const aFileName, aValue: String);
  Var
    aFile : TextFile;
  Begin
    AssignFile( aFile, aFileName);
    try
      Rewrite( aFile);
      Write( aFile, aValue);
    Finally
      CloseFile( aFile);
    End;
  End;

  Procedure StringToFileAppend( Const aFileName, aValue: String);
  Var
    aFile : TextFile;
  Begin
    AssignFile( aFile, aFileName);
    try
      if FileExists( aFileName)
      then Append ( aFile)
      else rewrite( aFile);
      Write( aFile, aValue);
    Finally
      CloseFile( aFile);
    End;
  End;

// /////////////////////////////////////////////////////////////////////////////

  Function  QueryUserName: String;
  Var
    aData : Array[ 0 .. 255] Of Char;
    Size  : DWORD;
  Begin
    Size := 255;
    If Windows.GetUserName( aData, Size)
    Then Result := StrPas( aData)
    Else Result := '';
  End;


  Procedure ExploreFolder( aWindowHandle: THandle; Const aFolderName: String);
  Begin
    ShellExecute(
      aWindowHandle,
      'explore',
      PChar( aFolderName),
      Nil,
      Nil,
      SW_SHOWNORMAL
    );
  End;

  Procedure ShellOpen( aWindowHandle: THandle; Const aFileName: String);
  Begin
    ShellExecute(
      aWindowHandle,
      'open',
      PChar( aFileName),
      Nil,
      Nil,
      SW_HIDE
    );
  End;


// /////////////////////////////////////////////////////////////////////////////

  Procedure InitVersInfo;
  Const
    InfoStr : Array[ 1 .. InfoNum] Of String = (
      'CompanyName',
      'FileDescription',
      'FileVersion',
      'InternalName',
      'LegalCopyRight',
      'LegalTradeMarks',
      'OriginalFileName',
      'ProductName',
      'ProductVersion',
      'Comments',
      'Phone',
      'Fax',
      'Address1',
      'Address2',
      'email',
      'web'
    );
  Var
    S      : String;
    n      : DWORD;
    Len    : DWORD;
    Buf    : PChar;
    aValue : PChar;
    i      : Integer;
  Begin
    S := Application.ExeName;
    n := GetFileVersionInfoSize( PChar( S), n);
    If n > 0
    Then Begin
      Buf := AllocMem( n);
      Try
        GetFileVersionInfo( PChar( S), 0, n, Buf);
        For i := 1 To InfoNum Do
          If
            VerQueryValue(
              Buf,
              PChar( '\StringFileInfo\040904E4\' + InfoStr[ i]),
              Pointer( aValue),
              Len
            )
          Then FileInfo[ i] := aValue;
      Finally
        FreeMem( Buf, n);
      End;
    End;
  End;

{$WARN UNSAFE_CODE ON}
{$WARN UNSAFE_TYPE ON}



// Initialisation:
//   get version info from executable
//   etc.

Initialization

  InitModule;
  InitVersInfo;

Finalization

end.


